@@ -1,10 +1,17 @@
 from .errorHandler import UnauthorizedException, ForbiddenException, ApiException, ConflictException, \
     ValidationException, InternalException, NotFoundException
-import requests
 from typing_extensions import TypedDict
 from typing import Optional
 from ..metaApi.models import ExceptionMessage
 import json
+import asyncio
+import sys
+import httpx
+from httpx import HTTPError, Response
+
+
+if sys.version_info[0] == 3 and sys.version_info[1] >= 8 and sys.platform.startswith('win'):
+    asyncio.set_event_loop_policy(asyncio.WindowsSelectorEventLoopPolicy())
 
 
 class RequestOptions(TypedDict):
@@ -27,7 +34,7 @@ class HttpClient:
         """
         self._timeout = timeout
 
-    async def request(self, options: RequestOptions) -> requests.Response:
+    async def request(self, options: RequestOptions) -> Response:
         """Performs a request. Response errors are returned as ApiError or subclasses.
 
         Args:
@@ -44,33 +51,28 @@ class HttpClient:
                     response = response.json()
                 except Exception as err:
                     print('Error parsing json', err)
-        except requests.HTTPError as err:
+        except HTTPError as err:
             self._convert_error(err)
         return response
 
-    async def _make_request(self, options: RequestOptions) -> requests.Response:
-        req = requests.Request()
-        req.method = options['method'] if ('method' in options) else 'GET'
-        req.url = options['url']
-        if 'params' in options:
-            req.params = options['params']
-        if 'files' in options:
-            req.files = options['files']
-        if 'headers' in options:
-            req.headers = options['headers']
-        if 'body' in options:
-            req.json = options['body']
-        prepped = req.prepare()
-        s = requests.Session()
-        return s.send(prepped, timeout=self._timeout)
+    async def _make_request(self, options: RequestOptions) -> Response:
+        async with httpx.AsyncClient(timeout=self._timeout) as client:
+            method = options['method'] if ('method' in options) else 'GET'
+            url = options['url']
+            params = options['params'] if 'params' in options else None
+            files = options['files'] if 'files' in options else None
+            headers = options['headers'] if 'headers' in options else None
+            body = options['body'] if 'body' in options else None
+            req = client.build_request(method, url, params=params, files=files, headers=headers, json=body)
+            response = await client.send(req)
+            return response
 
-    def _convert_error(self, err: requests.HTTPError):
+    def _convert_error(self, err: HTTPError):
         try:
             response: ExceptionMessage or TypedDict = json.loads(err.response.text)
         except Exception:
             response = {}
-
-        err_message = response['message'] if 'message' in response else err.response.reason
+        err_message = response['message'] if 'message' in response else err.response.reason_phrase
         status = err.response.status_code
         if status == 400:
             details = response['details'] if 'details' in response else []
