@@ -5,11 +5,15 @@ import iso8601
 import random
 import string
 import pytz
+import re
 
 
-def date(date_time: str) -> datetime:
+def date(date_time: str or float or int) -> datetime:
     """Parses a date string into a datetime object."""
-    return iso8601.parse_date(date_time)
+    if isinstance(date_time, float) or isinstance(date_time, int):
+        return datetime.fromtimestamp(max(date_time, 100000)).astimezone(pytz.utc)
+    else:
+        return iso8601.parse_date(date_time)
 
 
 def format_date(date: datetime) -> str:
@@ -22,6 +26,29 @@ def random_id(length: int = 32) -> str:
     return ''.join(random.choice(string.ascii_lowercase) for i in range(length))
 
 
+def convert_iso_time_to_date(packet):
+    for field in packet:
+        value = packet[field]
+        if isinstance(value, str) and re.search('time|Time', field) and not \
+                re.search('brokerTime|BrokerTime', field):
+            packet[field] = date(value)
+        if isinstance(value, list):
+            for item in value:
+                convert_iso_time_to_date(item)
+        if isinstance(value, dict):
+            convert_iso_time_to_date(value)
+    if packet and 'timestamps' in packet:
+        for field in packet['timestamps']:
+            packet['timestamps'][field] = date(packet['timestamps'][field])
+    if packet and 'type' in packet and packet['type'] == 'prices':
+        if 'prices' in packet:
+            for price in packet['prices']:
+                if 'timestamps' in price:
+                    for field in price['timestamps']:
+                        if isinstance(price['timestamps'][field], str):
+                            price['timestamps'][field] = date(price['timestamps'][field])
+
+
 class MetatraderAccountInformation(TypedDict):
     """MetaTrader account information (see https://metaapi.cloud/docs/client/models/metatraderAccountInformation/)"""
 
